@@ -4,6 +4,7 @@
 # Parti is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
+import sys
 from wimpiggy.gobject_compat import import_gobject
 gobject = import_gobject()
 
@@ -59,7 +60,7 @@ class ClientSource(object):
         has_more = packet is not None and \
                 (bool(self._priority_packets) or bool(self._ordinary_packets) \
                  or self._mouse_position is not None)
-        return packet, has_more
+        return packet, None, None, has_more
 
 
 class XpraClientBase(gobject.GObject):
@@ -76,7 +77,7 @@ class XpraClientBase(gobject.GObject):
 
     def __init__(self, opts):
         gobject.GObject.__init__(self)
-        self.exit_code = 1
+        self.exit_code = None
         self.password_file = opts.password_file
         self.encoding = opts.encoding
         self.jpegquality = opts.jpegquality
@@ -114,14 +115,19 @@ class XpraClientBase(gobject.GObject):
         capabilities["encodings"] = ENCODINGS
         if self.jpegquality:
             capabilities["jpeg"] = self.jpegquality
+        capabilities["platform"] = sys.platform
         capabilities["raw_packets"] = True
+        capabilities["server-window-resize"] = True
+        capabilities["randr_notify"] = False    #only client.py cares about this
         return capabilities
 
     def send(self, packet):
-        self._protocol.source.queue_ordinary_packet(packet)
+        if self._protocol:
+            self._protocol.source.queue_ordinary_packet(packet)
 
     def send_now(self, packet):
-        self._protocol.source.queue_priority_packet(packet)
+        if self._protocol:
+            self._protocol.source.queue_priority_packet(packet)
 
     def cleanup(self):
         if self._protocol:
@@ -131,19 +137,23 @@ class XpraClientBase(gobject.GObject):
     def run(self):
         raise Exception("override me!")
 
-    def quit(self, *args):
+    def quit(self, exit_code=0):
+        if self.exit_code is None:
+            self.exit_code = exit_code
         raise Exception("override me!")
 
     def _process_disconnect(self, packet):
         log.error("server requested disconnect: %s", packet[1:])
-        self.quit()
-        return
+        self.quit(0)
+
+    def _process_connection_lost(self, packet):
+        log.error("Connection lost")
+        self.quit(1)
 
     def _process_challenge(self, packet):
         if not self.password_file:
             log.error("password is required by the server")
-            self.exit_code = 2
-            self.quit()
+            self.quit(2)
             return
         import hmac
         try:
@@ -151,8 +161,7 @@ class XpraClientBase(gobject.GObject):
             password = passwordFile.read()
         except IOError, e:
             log.error("failed to open password file %s: %s", self.password_file, e)
-            self.exit_code = 3
-            self.quit()
+            self.quit(3)
             return
         salt = packet[1]
         challenge_response = hmac.HMAC(password, salt)
@@ -166,17 +175,13 @@ class XpraClientBase(gobject.GObject):
         self._protocol.raw_packets = bool(capabilities.get("raw_packets", False))
         self._remote_version = capabilities.get("version") or capabilities.get("__prerelease_version")
         if not is_compatible_with(self._remote_version):
-            self.quit(1)
+            self.quit(4)
 
     def _process_set_deflate(self, packet):
         #this tell us the server has set its compressor
         #(the decompressor has been enabled - see protocol)
         log.debug("set_deflate: %s", packet[1:])
 
-    def _process_connection_lost(self, packet):
-        log.error("Connection lost")
-        self.quit()
-
     def _process_gibberish(self, packet):
         (_, data) = packet
         log.info("Received uninterpretable nonsense: %s", repr(data))
@@ -203,6 +208,19 @@ class GLibXpraClient(XpraClientBase):
         self.ready(conn)
         self.send_hello()
 
+    def init_packet_handlers(self):
+        XpraClientBase.init_packet_handlers(self)
+        def noop(*args):
+            log("ignoring packet: %s", args)
+        #ignore the following packet types without error:
+        for t in ["new-window", "new-override-redirect",
+                  "draw", "cursor", "bell",
+                  "notify_show", "notify_close",
+                  "ping", "ping_echo",
+                  "window-metadata", "configure-override-redirect",
+                  "lost-window"]:
+            self._packet_handlers[t] = noop
+
     def run(self):
         import glib
         try:
@@ -224,13 +242,12 @@ class GLibXpraClient(XpraClientBase):
         capabilities["keyboard"] = False
         return capabilities
 
-    def quit(self, *args):
+    def quit(self, exit_code):
+        if self.exit_code is None:
+            self.exit_code = exit_code
         self.cleanup()
         gobject.timeout_add(50, self.glib_mainloop.quit)
 
-    def _process_connection_lost(self, packet):
-        log("Connection lost")
-        self.quit()
 
 class ScreenshotXpraClient(GLibXpraClient):
     """ This client does one thing only:
@@ -243,7 +260,7 @@ class ScreenshotXpraClient(GLibXpraClient):
         def screenshot_timeout(*args):
             self.exit_code = 1
             log.error("timeout: did not receive the screenshot")
-            self.quit()
+            self.quit(5)
         gobject.timeout_add(10*1000, screenshot_timeout)
         GLibXpraClient.__init__(self, conn, opts)
 
@@ -254,11 +271,10 @@ class ScreenshotXpraClient(GLibXpraClient):
         f.write(img_data)
         f.close()
         log.info("screenshot %sx%s saved to: %s", w, h, self.screenshot_filename)
-        self.exit_code = 0
-        self.quit()
+        self.quit(0)
 
     def init_packet_handlers(self):
-        XpraClientBase.init_packet_handlers(self)
+        GLibXpraClient.init_packet_handlers(self)
         self._packet_handlers["screenshot"] = self._process_screenshot
 
     def make_hello(self, challenge_response=None):
@@ -276,7 +292,7 @@ class InfoXpraClient(GLibXpraClient):
         def info_timeout(*args):
             self.exit_code = 1
             log.error("timeout: did not receive the info")
-            self.quit()
+            self.quit(5)
         gobject.timeout_add(10*1000, info_timeout)
         GLibXpraClient.__init__(self, conn, opts)
 
@@ -284,10 +300,10 @@ class InfoXpraClient(GLibXpraClient):
         log.debug("process_hello: %s", packet)
         props = packet[1]
         if props:
-            for k,v in props.items():
+            for k in sorted(props.keys()):
+                v = props.get(k)
                 log.info("%s=%s", k, v)
-        self.exit_code = 0
-        self.quit()
+        self.quit(0)
 
     def make_hello(self, challenge_response=None):
         capabilities = GLibXpraClient.make_hello(self, challenge_response)
@@ -303,9 +319,8 @@ class VersionXpraClient(GLibXpraClient):
 
     def __init__(self, conn, opts):
         def version_timeout(*args):
-            self.exit_code = 1
             log.error("timeout: did not receive the version")
-            self.quit()
+            self.quit(5)
         gobject.timeout_add(10*1000, version_timeout)
         GLibXpraClient.__init__(self, conn, opts)
 
@@ -313,8 +328,7 @@ class VersionXpraClient(GLibXpraClient):
         log.debug("process_hello: %s", packet)
         props = packet[1]
         log.info("%s" % props.get("version"))
-        self.exit_code = 0
-        self.quit()
+        self.quit(0)
 
     def make_hello(self, challenge_response=None):
         capabilities = GLibXpraClient.make_hello(self, challenge_response)
@@ -328,12 +342,10 @@ class StopXpraClient(GLibXpraClient):
 
     def __init__(self, conn, opts):
         def stop_timeout(*args):
-            self.exit_code = 1
             log.error("timeout: server did not disconnect us")
-            self.quit()
+            self.quit(5)
         gobject.timeout_add(5*1000, stop_timeout)
         GLibXpraClient.__init__(self, conn, opts)
 
     def _process_hello(self, packet):
         self.send(["shutdown-server"])
-        self.exit_code = 0
