@@ -98,6 +98,8 @@ class Compressible(object):
     def __init__(self, datatype, data):
         self.datatype = datatype
         self.data = data
+    def __len__(self):
+        return len(self.data)
 
 class Protocol(object):
     CONNECTION_LOST = "connection-lost"
@@ -126,16 +128,14 @@ class Protocol(object):
         self._compressor = None
         self._decompressor = zlib.decompressobj()
         self._compression_level = 0
-        self._write_thread = Thread(target=self._write_thread_loop)
-        self._write_thread.name = "write_loop"
-        self._write_thread.daemon = True
+        def make_daemon_thread(target, name):
+            daemon_thread = Thread(target=target, name=name)
+            daemon_thread.setDaemon(True)
+            return daemon_thread
         self._write_lock = Lock()
-        self._read_thread = Thread(target=self._read_thread_loop)
-        self._read_thread.name = "read_loop"
-        self._read_thread.daemon = True
-        self._read_parser_thread = Thread(target=self._read_parse_thread_loop)
-        self._read_parser_thread.name = "read_parse_loop"
-        self._read_parser_thread.deamon = True
+        self._write_thread = make_daemon_thread(self._write_thread_loop, "write_loop")
+        self._read_thread = make_daemon_thread(self._read_thread_loop, "read_loop")
+        self._read_parser_thread = make_daemon_thread(self._read_parse_thread_loop, "read_parse_loop")
 
     def __str__(self):
         return "Protocol(%s)" % self._conn
@@ -156,7 +156,7 @@ class Protocol(object):
             self._flush_one_packet_into_buffer()
         return False
 
-    def _queue_write(self, data, flush=False):
+    def _queue_write(self, data, start_cb=None, end_cb=None, flush=False):
         """
             This method should be called with _write_lock held
         """
@@ -164,16 +164,16 @@ class Protocol(object):
             return
         if self.raw_packets or self._compressor is None:
             #raw packets are compressed individually, without the header
-            self._write_queue.put(data)
+            self._write_queue.put((data, start_cb, end_cb))
             return
         c = self._compressor.compress(data)
         if c:
-            self._write_queue.put(c)
+            self._write_queue.put((c, None, None))
         if not flush:
             return
         c = self._compressor.flush(zlib.Z_SYNC_FLUSH)
         if c:
-            self._write_queue.put(c)
+            self._write_queue.put((c, start_cb, end_cb))
 
     def verify_packet(self, packet):
         """ look for None values which may have caused the packet to fail encoding """
@@ -205,9 +205,9 @@ class Protocol(object):
     def _flush_one_packet_into_buffer(self):
         if not self.source:
             return
-        packet, self._source_has_more = self.source.next_packet()
+        packet, start_send_cb, end_send_cb, self._source_has_more = self.source.next_packet()
         if packet is not None:
-            self._add_packet_to_queue(packet)
+            self._add_packet_to_queue(packet, start_send_cb, end_send_cb)
 
     def encode(self, packet):
         """
@@ -257,12 +257,13 @@ class Protocol(object):
         packets.append((0, True, main_packet))
         return packets
 
-    def _add_packet_to_queue(self, packet):
+    def _add_packet_to_queue(self, packet, start_send_cb=None, end_send_cb=None):
         packets = self.encode(packet)
         if not self.raw_packets:
             assert len(packets)==1
         try:
             self._write_lock.acquire()
+            counter = 0
             for index,compress,data in packets:
                 if self.raw_packets:
                     if compress and self._compression_level>0:
@@ -279,12 +280,20 @@ class Protocol(object):
                     assert index==0
                     l = len(data)
                     header = ("PS%014d" % l).encode('latin1')
+                scb, ecb = None, None
+                #fire the start_send_callback just before the first packet is processed:
+                if counter==0:
+                    scb = start_send_cb
+                #fire the end_send callback when the last packet (index==0) makes it out:
+                if index==0:
+                    ecb = end_send_cb
                 if l<4096 and sys.version<'3':
                     #send size and data together (low copy overhead):
-                    self._queue_write(header+data, True)
+                    self._queue_write(header+data, scb, ecb, True)
                 else:
                     self._queue_write(header)
-                    self._queue_write(data, True)
+                    self._queue_write(data, scb, ecb, True)
+                counter += 1
         finally:
             if packet[0]=="set_deflate":
                 level = packet[1]
@@ -300,24 +309,31 @@ class Protocol(object):
     def _write_thread_loop(self):
         try:
             while True:
-                buf = self._write_queue.get()
+                item = self._write_queue.get()
                 # Used to signal that we should exit:
-                if buf is None:
+                if item is None:
                     log("write thread: empty marker, exiting")
                     break
+                buf, start_cb, end_cb = item
                 try:
+                    if start_cb:
+                        start_cb(self.output_bytecount)
                     while buf and not self._closed:
                         written = untilConcludes(self._conn.write, buf)
                         if written:
                             buf = buf[written:]
                             self.output_raw_packetcount += 1
                             self.output_bytecount += written
+                    if end_cb:
+                        end_cb(self.output_bytecount)
                 except (OSError, IOError, socket.error), e:
                     self._call_connection_lost("Error writing to connection: %s" % e)
                     break
                 except TypeError:
-                    assert self._closed
-                    break
+                    #can happen during close(), in which case we just ignore:
+                    if self._closed:
+                        break
+                    raise
                 if self._write_queue.empty():
                     gobject.idle_add(self._maybe_queue_more_writes)
         finally:
@@ -369,128 +385,131 @@ class Protocol(object):
             to ensure we enable compression synchronously within the thread.
             (this due for removal when we drop old protocol support)
         """
-        try:
-            read_buffer = None
-            current_packet_size = -1
-            packet_index = 0
-            compression_level = False
-            raw_packets = {}
+        read_buffer = None
+        current_packet_size = -1
+        packet_index = 0
+        compression_level = False
+        raw_packets = {}
+        while not self._closed:
+            buf = self._read_queue.get()
+            if not buf:
+                log("read thread: empty marker, exiting")
+                gobject.idle_add(self.close)
+                return
+            #this is the old/unconditional compression code (to be removed):
+            if not self.raw_packets and self._compression_level>0:
+                buf = self._decompressor.decompress(buf)
+            if read_buffer:
+                read_buffer = read_buffer + buf
+            else:
+                read_buffer = buf
+            bl = len(read_buffer)
             while not self._closed:
-                buf = self._read_queue.get()
-                if not buf:
-                    log("read thread: empty marker, exiting")
-                    gobject.idle_add(self.close)
-                    return
-                #this is the old/unconditional compression code (to be removed):
-                if not self.raw_packets and self._compression_level>0:
-                    buf = self._decompressor.decompress(buf)
-                if read_buffer:
-                    read_buffer = read_buffer + buf
-                else:
-                    read_buffer = buf
                 bl = len(read_buffer)
-                while not self._closed:
-                    bl = len(read_buffer)
-                    if bl<=0:
+                if bl<=0:
+                    break
+                if current_packet_size<0:
+                    if read_buffer[0] not in ["P", ord("P")]:
+                        return self._call_connection_lost("invalid packet header: ('%s...'), not an xpra client?" % read_buffer[:32])
+                    if bl<2:
                         break
-                    if current_packet_size<0:
-                        if read_buffer[0] not in ["P", ord("P")]:
-                            return self._call_connection_lost("invalid packet header, not an xpra client?")
-                        if bl<2:
+                    if read_buffer[1] in ["S", ord("S")]:
+                        #old packet format: "PS%02d%012d" - 16 bytes
+                        if bl<16:
                             break
-                        if read_buffer[1] in ["S", ord("S")]:
-                            #old packet format: "PS%02d%012d" - 16 bytes
-                            if bl<16:
-                                break
-                            current_packet_size = int(read_buffer[2:16])
-                            packet_index = 0
-                            compression_level = 0
-                            read_buffer = read_buffer[16:]
-                        else:
-                            #new packet format: struct.pack('cBBBL', ...) - 8 bytes
-                            if bl<8:
-                                break
-                            try:
-                                (_, _, compression_level, packet_index, current_packet_size) = struct.unpack_from('!cBBBL', read_buffer)
-                            except Exception, e:
-                                raise Exception("invalid packet format: %s", e)
-                            read_buffer = read_buffer[8:]
-                        bl = len(read_buffer)
-
-                    if current_packet_size>self.max_packet_size:
-                        #this packet is seemingly too big, but check again from the main UI thread
-                        #this gives 'set_max_packet_size' a chance to run
-                        def check_packet_size(size_to_check, packet_header):
-                            log("check_packet_size(%s, %s) limit is %s", size_to_check, packet_header, self.max_packet_size)
-                            if size_to_check>self.max_packet_size:
-                                return self._call_connection_lost("invalid packet: size requested is %s (maximum allowed is %s - packet header: '%s'), dropping this connection!" %
-                                                              (size_to_check, self.max_packet_size, packet_header))
-                        gobject.timeout_add(1000, check_packet_size, current_packet_size, read_buffer[:32])
-
-                    if current_packet_size>0 and bl<current_packet_size:
-                        # incomplete packet
-                        break
-
-                    #chop this packet from the buffer:
-                    if len(read_buffer)==current_packet_size:
-                        raw_string = read_buffer
-                        read_buffer = ''
-                    else:
-                        raw_string = read_buffer[:current_packet_size]
-                        read_buffer = read_buffer[current_packet_size:]
-                    if compression_level>0:
-                        raw_string = self._decompressor.decompress(raw_string)
-                    if sys.version>='3':
-                        raw_string = raw_string.decode("latin1")
-
-                    if packet_index>0:
-                        #raw packet, store it and continue:
-                        raw_packets[packet_index] = raw_string
-                        current_packet_size = -1
+                        current_packet_size = int(read_buffer[2:16])
                         packet_index = 0
-                        continue
-                    result = None
-                    try:
-                        #final packet (packet_index==0), decode it:
-                        result = bdecode(raw_string)
-                    except ValueError, e:
-                        import traceback
-                        traceback.print_exc()
-                        log.error("value error reading packet: %s", e)
-                        if self._closed:
-                            return
-                        def gibberish(buf):
-                            # Peek at the data we got, in case we can make sense of it:
-                            self._process_packet([Protocol.GIBBERISH, buf])
-                            # Then hang up:
-                            return self._connection_lost("gibberish received: %s, packet index=%s, packet size=%s, buffer size=%s, error=%s" % (repr_ellipsized(raw_string), packet_index, current_packet_size, bl, e))
-                        gobject.idle_add(gibberish, raw_string)
-                        return
+                        compression_level = 0
+                        read_buffer = read_buffer[16:]
+                    else:
+                        #new packet format: struct.pack('cBBBL', ...) - 8 bytes
+                        if bl<8:
+                            break
+                        try:
+                            (_, _, compression_level, packet_index, current_packet_size) = struct.unpack_from('!cBBBL', read_buffer)
+                        except Exception, e:
+                            raise Exception("invalid packet format: %s", e)
+                        read_buffer = read_buffer[8:]
+                    bl = len(read_buffer)
 
+                if current_packet_size>self.max_packet_size:
+                    #this packet is seemingly too big, but check again from the main UI thread
+                    #this gives 'set_max_packet_size' a chance to run
+                    def check_packet_size(size_to_check):
+                        log("check_packet_size(%s) limit is %s", size_to_check, self.max_packet_size)
+                        if size_to_check>self.max_packet_size:
+                            return self._call_connection_lost("invalid packet: size requested is %s (maximum allowed is %s), dropping this connection!" %
+                                                              (size_to_check, self.max_packet_size))
+                    gobject.idle_add(check_packet_size, current_packet_size)
+
+                if current_packet_size>0 and bl<current_packet_size:
+                    # incomplete packet, wait for the rest to arrive
+                    break
+
+                #chop this packet from the buffer:
+                if len(read_buffer)==current_packet_size:
+                    raw_string = read_buffer
+                    read_buffer = ''
+                else:
+                    raw_string = read_buffer[:current_packet_size]
+                    read_buffer = read_buffer[current_packet_size:]
+                if compression_level>0:
+                    raw_string = self._decompressor.decompress(raw_string)
+                if sys.version>='3':
+                    raw_string = raw_string.decode("latin1")
+
+                if self._closed:
+                    return
+                if packet_index>0:
+                    #raw packet, store it and continue:
+                    raw_packets[packet_index] = raw_string
                     current_packet_size = -1
-                    if result is None or self._closed:
-                        break
-                    packet, l = result
-                    #add any raw packets back into it:
-                    if raw_packets:
-                        for index,raw_data in raw_packets.items():
-                            #replace placeholder with the raw_data packet data:
-                            packet[index] = raw_data
-                        raw_packets = {}
-                    gobject.idle_add(self._process_packet, packet)
-                    assert l==len(raw_string)
-                    if packet[0]=="set_deflate":
-                        level = packet[1]
-                        if level!=self._compression_level:
-                            log("set_deflate packet, changing compressor to level=%s", level)
-                            previous_level = self._compression_level
-                            self._compression_level = level
-                            if level>0:
-                                if previous_level==0 and not self.raw_packets:
-                                    # deflate was just enabled: so decompress the unprocessed data:
-                                    read_buffer = self._decompressor.decompress(read_buffer)
-        finally:
-            log("read parse thread: ended")
+                    packet_index = 0
+                    continue
+                result = None
+                try:
+                    #final packet (packet_index==0), decode it:
+                    result = bdecode(raw_string)
+                except ValueError, e:
+                    import traceback
+                    traceback.print_exc()
+                    log.error("value error reading packet: %s", e)
+                    if self._closed:
+                        return
+                    def gibberish(buf):
+                        # Peek at the data we got, in case we can make sense of it:
+                        self._process_packet([Protocol.GIBBERISH, buf])
+                        # Then hang up:
+                        return self._connection_lost("gibberish received: %s, packet index=%s, packet size=%s, buffer size=%s, error=%s" % (repr_ellipsized(raw_string), packet_index, current_packet_size, bl, e))
+                    gobject.idle_add(gibberish, raw_string)
+                    return
+
+                if self._closed:
+                    return
+                current_packet_size = -1
+                if result is None:
+                    break
+                packet, l = result
+                #add any raw packets back into it:
+                if raw_packets:
+                    for index,raw_data in raw_packets.items():
+                        #replace placeholder with the raw_data packet data:
+                        packet[index] = raw_data
+                    raw_packets = {}
+                gobject.idle_add(self._process_packet, packet)
+                assert l==len(raw_string)
+                #special case: we can't wait for idle_add to make the call...
+                #(this will be removed in 0.5 in favour of the per-packet compression header)
+                if packet[0]=="set_deflate":
+                    level = packet[1]
+                    if level!=self._compression_level:
+                        log("set_deflate packet, changing compressor to level=%s", level)
+                        previous_level = self._compression_level
+                        self._compression_level = level
+                        if level>0:
+                            if previous_level==0 and not self.raw_packets:
+                                # deflate was just enabled: so decompress the unprocessed data:
+                                read_buffer = self._decompressor.decompress(read_buffer)
 
     def _process_packet(self, decoded):
         if self._closed:
