@@ -195,6 +195,11 @@ def run_server(parser, opts, mode, xpra_file, extra_args):
     if len(extra_args) != 1:
         parser.error("need exactly 1 extra argument")
     display_name = extra_args.pop(0)
+    if display_name==":0":
+        print("WARNING:")
+        print("You are attempting to run the xpra server against the default X11 display '%s'." % display_name)
+        print("This is generally not what you want.")
+        print("You should probably use a higher display number just to avoid any confusion (and also this warning message).")
 
     if opts.exit_with_children and not opts.children:
         print("--exit-with-children specified without any children to spawn; exiting immediately")
@@ -276,7 +281,7 @@ def run_server(parser, opts, mode, xpra_file, extra_args):
             tcp_socket = create_tcp_socket(parser, opts.bind_tcp)
             sockets.append(tcp_socket)
             def cleanup_tcp_socket():
-                print("closing tcp socket")
+                print("closing tcp socket %s" % opts.bind_tcp)
                 try:
                     tcp_socket.close()
                 except:
@@ -307,14 +312,12 @@ def run_server(parser, opts, mode, xpra_file, extra_args):
 
     if not clobber:
         # We need to set up a new server environment
-        xauthority = os.environ.get("XAUTHORITY",
-                                    os.path.expanduser("~/.Xauthority"))
+        xauthority = os.environ.get("XAUTHORITY", os.path.expanduser("~/.Xauthority"))
         xvfb_cmd = opts.xvfb.replace("$XAUTHORITY", xauthority).split()
         xvfb_executable = xvfb_cmd[0]
         xvfb_cmd[0] = "%s-for-Xpra-%s" % (xvfb_executable, display_name)
         try:
-            xvfb = subprocess.Popen(xvfb_cmd+[display_name],
-                                     executable=xvfb_executable)
+            xvfb = subprocess.Popen(xvfb_cmd+[display_name], executable=xvfb_executable)
         except OSError, e:
             sys.stderr.write("Error starting Xvfb: %s\n" % (e,))
             return  1
@@ -338,7 +341,7 @@ def run_server(parser, opts, mode, xpra_file, extra_args):
         sys.stderr.write("\n")
         if instance_exists:
             sys.stderr.write("There is an X11 server already running on display %s:\n" % display_name)
-            sys.stderr.write("You may want to use:\n" % display_name)
+            sys.stderr.write("You may want to use:\n")
             sys.stderr.write("  'xpra upgrade' if an instance of xpra is still connected to it\n")
             sys.stderr.write("  'xpra --use-display start' to connect xpra to an existing X11 server only\n")
         return True
@@ -370,6 +373,8 @@ def run_server(parser, opts, mode, xpra_file, extra_args):
         if xvfb_error(True):
             return  1
         xvfb_pid = xvfb.pid
+        if xvfb_pid is not None:
+            save_pid(xvfb_pid)
 
     def kill_xvfb():
         # Close our display(s) first, so the server dying won't kill us.
@@ -380,9 +385,6 @@ def run_server(parser, opts, mode, xpra_file, extra_args):
             os.kill(xvfb_pid, signal.SIGTERM)
     _cleanups.append(kill_xvfb)
 
-    if xvfb_pid is not None:
-        save_pid(xvfb_pid)
-
     from xpra.server import can_run_server, XpraServer
     if not can_run_server():
         return  1
