@@ -20,8 +20,8 @@ class ClipboardProtocolHelper(ClipboardProtocolHelperBase):
     def __init__(self, send_packet_cb):
         ClipboardProtocolHelperBase.__init__(self, send_packet_cb, ["CLIPBOARD", "PRIMARY", "SECONDARY"])
 
-    def _do_munge_raw_selection_to_wire(self, target, dtype, dformat, data):
-        if dformat == 32 and dtype in ("ATOM", "ATOM_PAIR"):
+    def _do_munge_raw_selection_to_wire(self, target, ype, format, data):
+        if format == 32 and type in ("ATOM", "ATOM_PAIR"):
             # Convert to strings and send that. Bizarrely, the atoms are
             # not actual X atoms, but an array of GdkAtom's reinterpreted
             # as a byte buffer.
@@ -31,12 +31,12 @@ class ClipboardProtocolHelper(ClipboardProtocolHelperBase):
                 atom_names.remove("SAVE_TARGETS")
                 atom_names.remove("COMPOUND_TEXT")
             return ("atoms", atom_names)
-        return ClipboardProtocolHelperBase._do_munge_raw_selection_to_wire(self, target, dtype, dformat, data)
+        return ClipboardProtocolHelperBase._do_munge_raw_selection_to_wire(self, target, type, format, data)
 
-    def _munge_wire_selection_to_raw(self, encoding, dtype, dformat, data):
+    def _munge_wire_selection_to_raw(self, encoding, datatype, format, data):
         if encoding == "atoms":
             import gtk.gdk
             gdk_atoms = [gtk.gdk.atom_intern(a) for a in data]
             atom_array = gdk_atom_array_from_gdk_atom_objects(gdk_atoms)
-            return struct.pack("=" + "L" * len(atom_array), *atom_array)
-        return ClipboardProtocolHelperBase._munge_wire_selection_to_raw(self, encoding, dtype, dformat, data)
+            return struct.pack("=" + "I" * len(atom_array), *atom_array)
+        return ClipboardProtocolHelperBase._munge_wire_selection_to_raw(self, encoding, datatype, format, data)
